var __assign = (this && this.__assign) || function () {
    __assign = Object.assign || function(t) {
        for (var s, i = 1, n = arguments.length; i < n; i++) {
            s = arguments[i];
            for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p))
                t[p] = s[p];
        }
        return t;
    };
    return __assign.apply(this, arguments);
};
var __read = (this && this.__read) || function (o, n) {
    var m = typeof Symbol === "function" && o[Symbol.iterator];
    if (!m) return o;
    var i = m.call(o), r, ar = [], e;
    try {
        while ((n === void 0 || n-- > 0) && !(r = i.next()).done) ar.push(r.value);
    }
    catch (error) { e = { error: error }; }
    finally {
        try {
            if (r && !r.done && (m = i["return"])) m.call(i);
        }
        finally { if (e) throw e.error; }
    }
    return ar;
};
import { useCallback, useEffect, useMemo, useRef, useState } from 'react';
import { formatTime as formatTimeByString, isBrowser, getTimeByString, getformatTimeInfo, invariant } from '../utils';
export default function useCountDown(_a) {
    var onStart = _a.onStart, onStep = _a.onStep, onFinished = _a.onFinished, step = _a.step, _b = _a.format, format = _b === void 0 ? 'hh:mm:ss' : _b, endTime = _a.endTime, total = _a.total, _c = _a.autoRun, autoRun = _c === void 0 ? true : _c;
    invariant(!!endTime || !!total, 'endTime and total need least one');
    invariant(!endTime || !total, 'endTime and total need only one');
    var intervalRef = useRef({ timer: undefined, type: '' });
    var timerRef = useRef(null);
    var statusRef = useRef('idle');
    var endTimeRef = useRef(typeof endTime === 'string' ? getTimeByString(endTime) : endTime || Date.now() + (total || 0));
    var _d = __read(useState({
        status: 'idle',
        endTime: endTimeRef.current,
        countdown: 0,
    }), 2), _e = _d[0], endTimeState = _e.endTime, status = _e.status, countdown = _e.countdown, setState = _d[1];
    var timeoutType = useRef(getTimeoutType(step));
    var formatedCountdown = useMemo(function () { return formatTime(countdown, format); }, [countdown, format]);
    var timeoutFn = useMemo(function () { return getTimeoutMethod(step); }, [step]);
    intervalRef.current.type = timeoutFn.type;
    // 停止倒计时
    var stop = useCallback(function () {
        clearTimer(intervalRef.current.timer, timeoutType.current);
        clearTimeout(timerRef.current);
        setState(function (prev) { return (__assign(__assign({}, prev), { countdown: 0, status: 'finished' })); });
        statusRef.current = 'finished';
        onFinished === null || onFinished === void 0 ? void 0 : onFinished();
    }, [onFinished, setState]);
    // 开始倒计时
    var start = useCallback(function () {
        clearTimer(intervalRef.current.timer, timeoutType.current);
        onStart === null || onStart === void 0 ? void 0 : onStart();
        endTimeRef.current = !endTime ? Date.now() + (total || 0) : endTimeRef.current;
        setState(function (prev) { return (__assign(__assign({}, prev), { status: 'running', endTime: endTimeRef.current })); });
        statusRef.current = 'running';
        var running = function () {
            // 每次都取当前时间比较，而非减去step的时间，防止进入后台等造成不准确的情况
            var currentCountDown = Math.max(endTimeRef.current - Date.now(), 0);
            var formatedProgress = formatTime(currentCountDown, format);
            setState(function (prev) { return (__assign(__assign({}, prev), { countdown: currentCountDown })); });
            onStep === null || onStep === void 0 ? void 0 : onStep(currentCountDown, formatedProgress);
            currentCountDown === 0 && stop();
            return currentCountDown;
        };
        timerRef.current = setTimeout(function () {
            var loop = function () {
                clearTimer(intervalRef.current.timer, timeoutType.current);
                clearTimeout(timerRef.current);
                var progress = running();
                if (progress > 0 && statusRef.current === 'running') {
                    intervalRef.current.timer = timeoutFn(loop);
                }
            };
            loop();
        }, 0);
    }, [endTime, format, onStart, onStep, stop, total, timeoutFn]);
    useEffect(function () {
        autoRun && start();
        var interval = intervalRef.current;
        var timerType = timeoutType.current;
        return function () {
            clearTimer(interval.timer, timerType);
            timerRef.current && clearTimeout(timerRef.current);
        };
        // eslint-disable-next-line react-hooks/exhaustive-deps
    }, []);
    return __assign(__assign({ status: status, endTime: endTimeState, formatedCountdown: formatedCountdown, countdown: countdown }, getformatTimeInfo(countdown)), { start: start, stop: stop });
}
function formatTime(timestamp, format) {
    // eslint-disable-next-line no-nested-ternary
    return typeof format === 'string'
        ? formatTimeByString(timestamp, format)
        : typeof format === 'function'
            ? format(timestamp)
            : '';
}
function clearTimer(timer, timeoutType) {
    if (!timer)
        return;
    if (timeoutType === 'raf') {
        try {
            var clearRaf = window.cancelAnimationFrame || window.webkitCancelAnimationFrame;
            clearRaf(timer);
        }
        catch (error) { }
        return;
    }
    if (timeoutType === 'timeout') {
        try {
            clearTimeout(timer);
        }
        catch (error) { }
        return;
    }
    try {
        clearInterval(timer);
    }
    catch (error) { }
}
function getTimeoutType(step) {
    return isBrowser && (window.requestAnimationFrame || window.webkitRequestAnimationFrame) && step < 17 ? 'raf' : 'timeout';
}
function getTimeoutMethod(step) {
    if (getTimeoutType(step) === 'raf') {
        return window.requestAnimationFrame || window.webkitRequestAnimationFrame;
    }
    var delay = (step && step >= 17) || !isBrowser ? step : 1000 / 60;
    var offset = 0; // 误差时间
    var count = 0;
    var nextDelay = delay;
    var start = Date.now();
    return function (callback) {
        nextDelay = delay - offset;
        return setTimeout(function () {
            offset = Date.now() - start - count * delay;
            count++;
            callback();
        }, nextDelay);
    };
}
