import { useEffect, createContext, useContext } from 'react';
import useRefCallback from '../useRefCallback';
import { EventBus } from './eventBus';
/**
 * 订阅当前事件的函数，回调函数传进来后就不会再改变，如果在react组件中中使用有可能会造成函数依赖值不是最新
 * 一般情况下hooks/组件中直接使用useSubscribe，外部js中订阅使用subscribe
 */
function subscribeOrigin(bus, type, callback, options) {
    if (options === void 0) { options = {}; }
    return options.once ? bus === null || bus === void 0 ? void 0 : bus.subscribeOnce(type, callback) : bus === null || bus === void 0 ? void 0 : bus.subscribe(type, callback);
}
function useSubscribeOrigin(bus, eventName, callback, options) {
    if (options === void 0) { options = {}; }
    var subscription = useRefCallback(callback);
    useEffect(function () {
        var unSubscribe = subscribeOrigin(bus, eventName, subscription, options);
        return function () {
            unSubscribe && unSubscribe();
        };
    }, []);
}
function useEventBus(bus, type) {
    return {
        publish: type ? bus.publish.bind(null, type) : bus.publish.bind(null),
        useSubscribe: type ? useSubscribeOrigin.bind(null, bus, type) : useSubscribeOrigin.bind(null, bus),
        subscribe: type ? subscribeOrigin.bind(null, bus, type) : subscribeOrigin.bind(null, bus),
    };
}
/**
 * 创建一个独立的eventBus实例
 * @returns
 */
export function createEventBus() {
    var bus = new EventBus();
    return {
        publish: bus === null || bus === void 0 ? void 0 : bus.publish.bind(null),
        useSubscribe: useSubscribeOrigin.bind(null, bus),
        subscribe: subscribeOrigin.bind(null, bus),
        useEventBus: useEventBus.bind(null, bus),
    };
}
/**
 * 可以生成一个用于上下文的EventBus及相关hooks
 */
export function createContextEventBus() {
    var contextState = createEventBus();
    var eventBusContext = createContext(contextState);
    var useContextEventBus = function () {
        return useContext(eventBusContext);
    };
    eventBusContext.useEventBus = useContextEventBus;
    return eventBusContext;
}
var _a = createEventBus(), publish = _a.publish, useEventBusWithoutContext = _a.useEventBus, useSubscribeWithoutContext = _a.useSubscribe;
export { useEventBusWithoutContext as default, EventBus, publish, useSubscribeWithoutContext as useSubscribe };
