function defaultDump(source) {
    if (typeof source === 'object' || Array.isArray(source)) {
        return JSON.parse(JSON.stringify(source));
    }
    return source;
}
function defaultParse(sourceDump) {
    try {
        return JSON.parse(sourceDump);
    }
    catch (error) {
        return sourceDump;
    }
}
var HistoryState = /** @class */ (function () {
    function HistoryState(options) {
        if (options === void 0) { options = {}; }
        this.last = {
            snapshot: undefined,
            timestamp: 0,
        };
        this.undoList = [];
        this.redoList = [];
        var max = options.max, _a = options.dump, dump = _a === void 0 ? defaultDump : _a, _b = options.parse, parse = _b === void 0 ? defaultParse : _b, _c = options.onUpdate, onUpdate = _c === void 0 ? function (nextState) { } : _c;
        this.dump = dump;
        this.parse = parse;
        this.onUpdate = onUpdate;
        this.push = this.push.bind(this);
        this.undo = this.undo.bind(this);
        this.redo = this.redo.bind(this);
        this.clear = this.clear.bind(this);
        this.reset = this.reset.bind(this);
        this.max = max;
    }
    Object.defineProperty(HistoryState.prototype, "canUndo", {
        get: function () {
            return this.undoList.length > 0;
        },
        enumerable: false,
        configurable: true
    });
    Object.defineProperty(HistoryState.prototype, "canRedo", {
        get: function () {
            return this.redoList.length > 0;
        },
        enumerable: false,
        configurable: true
    });
    Object.defineProperty(HistoryState.prototype, "history", {
        get: function () {
            return this.undoList;
        },
        enumerable: false,
        configurable: true
    });
    HistoryState.prototype.createHistoryItem = function (newSource) {
        var newHistoryItem = {
            snapshot: this.dump(newSource),
            timestamp: Date.now(),
        };
        return newHistoryItem;
    };
    HistoryState.prototype.undo = function () {
        if (this.canUndo) {
            var state = this.undoList.shift();
            if (state && this.last) {
                this.redoList.unshift(this.last);
                this._changeState(state);
            }
        }
    };
    HistoryState.prototype.redo = function () {
        if (this.canRedo) {
            var state = this.redoList.shift();
            this.undoList.unshift(this.last);
            this._changeState(state);
        }
    };
    HistoryState.prototype.clear = function () {
        this.undoList = [];
        this.redoList = [];
        this.onUpdate(this.source, this.last);
    };
    HistoryState.prototype.push = function (newSource) {
        var _a, _b;
        var isFirst = typeof this.source === 'undefined';
        this.source = newSource;
        if (isFirst) {
            this.undoList = [];
            this.redoList = [];
            var next = this.createHistoryItem(newSource);
            this.last = next;
            this.onUpdate(this.source, this.last);
        }
        else if (this.last && this.last.snapshot !== ((_b = (_a = this.undoList) === null || _a === void 0 ? void 0 : _a[0]) === null || _b === void 0 ? void 0 : _b.snapshot)) {
            this.undoList.unshift(this.last);
            if (this.max && this.undoList.length > this.max)
                this.undoList.splice(this.max, Infinity);
            if (this.redoList.length)
                this.redoList.splice(0, this.redoList.length);
            this.last = this.createHistoryItem(this.source);
            this._changeState(this.last);
        }
    };
    HistoryState.prototype.reset = function () {
        this._changeState(this.last);
    };
    HistoryState.prototype._changeState = function (record) {
        var next = this.parse(record.snapshot);
        this.source = next;
        this.last = record;
        this.onUpdate(next, record);
    };
    HistoryState.prototype.getHistory = function () {
        return this.history;
    };
    HistoryState.prototype.getUndoList = function () {
        return this.undoList;
    };
    HistoryState.prototype.getRedoList = function () {
        return this.redoList;
    };
    HistoryState.prototype.getCanUndo = function () {
        return this.canUndo;
    };
    HistoryState.prototype.getCanRedo = function () {
        return this.canRedo;
    };
    HistoryState.prototype.getSource = function () {
        return this.source;
    };
    HistoryState.prototype.getLast = function () {
        return this.last;
    };
    return HistoryState;
}());
export default HistoryState;
