var __assign = (this && this.__assign) || function () {
    __assign = Object.assign || function(t) {
        for (var s, i = 1, n = arguments.length; i < n; i++) {
            s = arguments[i];
            for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p))
                t[p] = s[p];
        }
        return t;
    };
    return __assign.apply(this, arguments);
};
var __read = (this && this.__read) || function (o, n) {
    var m = typeof Symbol === "function" && o[Symbol.iterator];
    if (!m) return o;
    var i = m.call(o), r, ar = [], e;
    try {
        while ((n === void 0 || n-- > 0) && !(r = i.next()).done) ar.push(r.value);
    }
    catch (error) { e = { error: error }; }
    finally {
        try {
            if (r && !r.done && (m = i["return"])) m.call(i);
        }
        finally { if (e) throw e.error; }
    }
    return ar;
};
var __spreadArray = (this && this.__spreadArray) || function (to, from, pack) {
    if (pack || arguments.length === 2) for (var i = 0, l = from.length, ar; i < l; i++) {
        if (ar || !(i in from)) {
            if (!ar) ar = Array.prototype.slice.call(from, 0, i);
            ar[i] = from[i];
        }
    }
    return to.concat(ar || Array.prototype.slice.call(from));
};
import { useCallback, useEffect, useMemo } from 'react';
import useObjectState from '../useObjectState';
import useAsync from '../useAsync';
export var defaultInitialQuery = {
    pageNo: 1,
    pageSize: 10,
    query: {},
};
var listViewReducers = {
    setQuery: function (state, payload) { return (__assign(__assign({}, state), { query: typeof payload === 'function' ? payload(state.query) : payload })); },
};
/**
 * 处理列表数据的hooks
 */
export default function useListViewData(fetchFn, options) {
    if (options === void 0) { options = {}; }
    var _a = options.isMerge, isMerge = _a === void 0 ? false : _a, _b = options.autoLoad, autoLoad = _b === void 0 ? false : _b, _c = options.initialQuery, initialQuery = _c === void 0 ? {} : _c;
    var initQuery = useMemo(function () { return (__assign(__assign({}, defaultInitialQuery), initialQuery)); }, [initialQuery]);
    var _d = __read(useAsync(fetchFn), 2), asyncFetchData = _d[0], loading = _d[1].loading;
    var _e = __read(useObjectState({
        hasMore: false,
        total: 0,
        pageTotal: 0,
        listData: [],
        query: initQuery, // 用于筛选的请求参数
    }, listViewReducers), 3), _f = _e[0], listData = _f.listData, query = _f.query, hasMore = _f.hasMore, total = _f.total, pageTotal = _f.pageTotal, setObjectState = _e[1], setQuery = _e[2].setQuery;
    // 获取数据, filter通过请求参数传入
    var loadData = useCallback(function (params) {
        if (params === void 0) { params = {}; }
        var currentQuery = __assign(__assign({}, query), params);
        return new Promise(function (resolve, reject) {
            asyncFetchData(currentQuery)
                .then(function (_a) {
                var data = _a.data, hasMorePage = _a.hasMore, total = _a.total;
                var pageTotal = +(total / currentQuery.pageSize).toFixed(0);
                setObjectState(function (prevState) { return ({
                    total: total,
                    pageTotal: pageTotal,
                    hasMore: hasMorePage !== null && hasMorePage !== void 0 ? hasMorePage : pageTotal > currentQuery.pageNo,
                    query: currentQuery,
                    listData: isMerge ? __spreadArray(__spreadArray([], __read(prevState.listData), false), __read(data), false) : data,
                }); });
                resolve('load data success');
            })
                .catch(function (e) {
                reject(new Error("load data error"));
            });
        });
    }, [asyncFetchData, isMerge, query, setObjectState]);
    useEffect(function () {
        autoLoad && loadData();
        // eslint-disable-next-line react-hooks/exhaustive-deps
    }, [autoLoad]);
    // 获取下一页数据
    var loadNextPage = useCallback(function () {
        return (hasMore &&
            loadData(__assign(__assign({}, query), { pageNo: +query.pageNo + 1 })));
    }, [loadData, query, hasMore]);
    // 获取上一页数据
    var loadPrevPage = useCallback(function () {
        return (query.pageNo - 1 > 0 &&
            loadData(__assign(__assign({}, query), { pageNo: query.pageNo - 1 })));
    }, [loadData, query]);
    /**
     * 清空筛选条件
     */
    var clearQuery = useCallback(function () {
        setQuery(initQuery);
    }, [initQuery, setQuery]);
    // 重载数据，即清空分页、查询条件重新请求
    var reloadData = useCallback(function () {
        setQuery(initQuery);
        return loadData(initQuery);
    }, [initQuery, loadData, setQuery]);
    return {
        current: query.pageNo,
        total: total,
        pageTotal: pageTotal,
        hasMore: hasMore,
        listData: listData,
        query: query,
        loading: loading,
        loadData: loadData,
        loadNextPage: loadNextPage,
        loadPrevPage: loadPrevPage,
        clearQuery: clearQuery,
        reloadData: reloadData,
        setQuery: setQuery,
    };
}
