interface CountDownOptions {
    /**
     * @description 倒计时时长，格式毫秒, 与endTime二者只能填一个
     */
    total?: number;
    /**
     * @description 结束时间，时间戳格式 或 "yyyy-mm-dd HH:MM:SS"
     */
    endTime?: number | string;
    /**
     * @description 展示的格式，如"hh:mm:ss", 或使用函数自行格式化
     */
    format?: string | ((progress: number) => string);
    /**
     * @description 倒计时间隔的时间，默认为1000，小于17时按浏览器帧率显示
     */
    step?: number;
    /**
     * @description 开始倒计时回调
     */
    onStart?: () => void;
    /**
     * @description 倒计时过程中回调
     */
    onStep?: (step: number, formatedProgress: string) => void;
    /**
     * @description 倒计时结束回调
     */
    onFinished?: () => void;
    /**
     * @description 自定义渲染
     */
    autoRun?: boolean;
}
export default function useCountDown({ onStart, onStep, onFinished, step, format, endTime, total, autoRun, }: CountDownOptions): {
    start: () => void;
    stop: () => void;
    day: number;
    hour: number;
    minute: number;
    second: number;
    millisecond: number;
    status: "idle" | "running" | "pause" | "resumed" | "finished";
    endTime: number;
    formatedCountdown: string;
    countdown: number;
};
export {};
