"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
var react_1 = require("react");
var useForceRender_1 = __importDefault(require("../useForceRender"));
var utils_1 = require("../utils");
/**
 * 计时hooks，可以用于天、时、分、秒、毫秒的计时
 */
function useCountup(_a) {
    var onStart = _a.onStart, onStep = _a.onStep, onFinished = _a.onFinished, step = _a.step, _b = _a.format, format = _b === void 0 ? 'dd hh:mm:ss.SSS' : _b, maxLength = _a.maxLength, _c = _a.autoRun, autoRun = _c === void 0 ? true : _c, _d = _a.loopWhenEnd, loopWhenEnd = _d === void 0 ? false : _d;
    var intervalRef = (0, react_1.useRef)({ timer: undefined, type: '' });
    var timerRef = (0, react_1.useRef)(null);
    var statusRef = (0, react_1.useRef)('idle');
    var runningListRef = (0, react_1.useRef)([]);
    var countUpRef = (0, react_1.useRef)(0);
    var forceUpdate = (0, useForceRender_1.default)();
    var timeoutFn = (0, react_1.useMemo)(function () { return getTimeoutMethod(step, typeof format === 'string' && /S+/gi.test(format)); }, [step]);
    intervalRef.current.type = timeoutFn.type;
    var setCountup = (0, react_1.useCallback)(function (value, shouldForceUpdate) {
        if (shouldForceUpdate === void 0) { shouldForceUpdate = false; }
        countUpRef.current = value;
        shouldForceUpdate && forceUpdate();
    }, []);
    var formatedCountup = formatTime(countUpRef.current, format);
    // 停止计时
    var stop = (0, react_1.useCallback)(function (reset) {
        if (reset === void 0) { reset = false; }
        (0, utils_1.clearTimer)(intervalRef.current.timer, intervalRef.current.type);
        clearTimeout(timerRef.current);
        statusRef.current = 'finished';
        onFinished && onFinished(countUpRef.current, formatTime(countUpRef.current, format));
        runningListRef.current = [];
        setCountup(reset ? 0 : countUpRef.current, true);
    }, [onFinished, format]);
    // 重置计时
    var reset = (0, react_1.useCallback)(function () {
        (0, utils_1.clearTimer)(intervalRef.current.timer, intervalRef.current.type);
        clearTimeout(timerRef.current);
        statusRef.current = 'idle';
        runningListRef.current = [];
        setCountup(0, true);
    }, []);
    var getCurrentCountup = (0, react_1.useCallback)(function (startTime) {
        // 每次都取当前时间比较，而非减去step的时间，防止进入后台等造成不准确的情况
        var currentCountup = Math.max(Date.now() - startTime, 0);
        var isOvertime = maxLength && currentCountup >= maxLength;
        if (isOvertime) {
            if (loopWhenEnd) {
                currentCountup = 0;
                startTime = Date.now();
            }
            else {
                currentCountup = maxLength;
            }
        }
        var formatedProgress = formatTime(currentCountup, format);
        setCountup(currentCountup, true);
        onStep && onStep(currentCountup, formatedProgress);
        // 超时并且不循环的话停止计时
        isOvertime && !loopWhenEnd && stop();
        return { countup: currentCountup, formatedCountup: formatedProgress };
    }, [maxLength, loopWhenEnd, onStep, format]);
    // 开始计时
    var start = (0, react_1.useCallback)(function () {
        onStart && onStart();
        var startTime = Date.now();
        var running = function () { return getCurrentCountup(startTime); };
        timerRef.current = setTimeout(function () {
            statusRef.current = 'running';
            runningListRef.current.push(running);
            var loop = function () {
                (0, utils_1.clearTimer)(intervalRef.current.timer, intervalRef.current.type);
                clearTimeout(timerRef.current);
                // const progress = running()
                // 维护一个running函数数组，用于暂停和恢复
                var current = runningListRef.current.splice(0, 1)[0];
                var progress = current === null || current === void 0 ? void 0 : current();
                if ((loopWhenEnd || progress.countup > 0) && statusRef.current === 'running') {
                    runningListRef.current.push(running);
                    intervalRef.current.timer = timeoutFn(loop);
                }
            };
            loop();
        }, 0);
    }, [onStart, getCurrentCountup, loopWhenEnd, timeoutFn]);
    (0, react_1.useEffect)(function () {
        autoRun && start();
        return function () {
            (0, utils_1.clearTimer)(intervalRef.current.timer, intervalRef.current.type);
            timerRef.current && clearTimeout(timerRef.current);
        };
    }, []);
    return {
        status: statusRef.current,
        formatedCountup: formatedCountup,
        countup: countUpRef.current,
        start: start,
        stop: stop,
        reset: reset,
        getCurrentCountup: getCurrentCountup,
    };
}
exports.default = useCountup;
function formatTime(timestamp, format) {
    // eslint-disable-next-line no-nested-ternary
    return typeof format === 'string'
        ? (0, utils_1.formatTime)(timestamp, format)
        : typeof format === 'function'
            ? format(timestamp)
            : '';
}
function getTimeoutMethod(step, useRaf) {
    var fn;
    if (utils_1.isBrowser &&
        (window.requestAnimationFrame || window.webkitRequestAnimationFrame) &&
        (step < 17 || useRaf)) {
        fn = window.requestAnimationFrame || window.webkitRequestAnimationFrame;
        fn.type = 'raf';
        return fn;
    }
    var delay = (step && step >= 17) || !utils_1.isBrowser ? step : 1000 / 60;
    var offset = 0; // 误差时间
    var count = 0;
    var nextDelay = delay;
    var start = Date.now();
    fn = function (callback) {
        nextDelay = delay - offset;
        return setTimeout(function () {
            offset = Date.now() - start - count * delay;
            count++;
            callback();
        }, nextDelay);
    };
    fn.type = 'timeout';
    return fn;
}
