"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.EventBus = void 0;
// 调用订阅的方法
function applySubscription(subscription, payload) {
    try {
        subscription(payload);
    }
    catch (e) {
        // 延迟抛出错误，防止阻塞了其他任务的执行
        setTimeout(function () {
            throw e;
        }, 0);
    }
}
var EventBus = /** @class */ (function () {
    function EventBus(options) {
        if (options === void 0) { options = {}; }
        var _this = this;
        this.eventContainer = new Map();
        this.cacheLatest = new Map();
        this.autoFireLatest = false;
        this.publish = function (type, payload) {
            var subscriptions = _this.eventContainer.get(type);
            _this.cacheLatest.set(type, {
                hasFired: true,
                payload: payload,
            });
            if (subscriptions) {
                subscriptions.forEach(function (subscription) { return applySubscription(subscription, payload); });
            }
        };
        this.subscribe = function (type, handler) {
            var subscriptions = _this.eventContainer.get(type) || new Set();
            subscriptions.add(handler);
            _this.eventContainer.set(type, subscriptions);
            // autoFireLatest 为 true时绑定后立即调用最近一次触发
            if (_this.autoFireLatest && _this.cacheLatest.has(type)) {
                applySubscription(handler, _this.cacheLatest.get(type).payload);
            }
            return _this.unSubscribe.bind(_this, type, handler);
        };
        this.unSubscribe = function (type, subscription) {
            var subscriptions = _this.eventContainer.get(type);
            subscriptions && subscriptions.delete(subscription);
            return _this;
        };
        this.autoFireLatest = options.autoFireLatest;
    }
    // 只订阅一次，自动销毁
    EventBus.prototype.subscribeOnce = function (type, handler) {
        var _this = this;
        var subscription = function () {
            var args = [];
            for (var _i = 0; _i < arguments.length; _i++) {
                args[_i] = arguments[_i];
            }
            handler.apply(null, args);
            _this.unSubscribe(type, subscription);
        };
        this.subscribe(type, subscription);
    };
    EventBus.prototype.clearSubscriptions = function (type) {
        var _this = this;
        this.eventContainer.forEach(function (subscriptions, eventType) {
            if (eventType === type && subscriptions.size) {
                _this.eventContainer.delete(eventType);
            }
        });
    };
    return EventBus;
}());
exports.EventBus = EventBus;
