import React from 'react';
import { EventBus, EventType, Subscription } from './eventBus';
interface Publish {
    (type: EventType, payload?: any): void;
    (payload?: any): void;
}
interface SubscribeOptions {
    once?: boolean;
}
interface UseSubscribe<T = any> {
    (type: EventType, callback: Subscription<T>, options?: SubscribeOptions): void;
}
interface Subscribe {
    <T>(type: EventType, callback: Subscription<T>, options?: SubscribeOptions): any;
    <T>(callback: Subscription<T>, options?: SubscribeOptions): any;
}
declare type UseEventBus = <T = any>(type?: EventType) => {
    publish: Publish;
    useSubscribe: T;
    subscribe: any;
};
interface EventBusContext<T> extends React.Context<{
    publish: Publish;
    useSubscribe: UseSubscribe<T>;
    subscribe: Subscribe;
    useEventBus: UseEventBus;
}> {
    useEventBus?: () => {
        publish: Publish;
        useSubscribe: UseSubscribe<T>;
        subscribe: Subscribe;
        useEventBus: UseEventBus;
    };
}
/**
 * 创建一个独立的eventBus实例
 * @returns
 */
export declare function createEventBus(): {
    publish: Publish;
    useSubscribe: UseSubscribe<any>;
    subscribe: Subscribe;
    useEventBus: UseEventBus;
};
/**
 * 可以生成一个用于上下文的EventBus及相关hooks
 */
export declare function createContextEventBus<T = any>(): EventBusContext<T>;
declare const publish: Publish, useEventBusWithoutContext: UseEventBus, useSubscribeWithoutContext: UseSubscribe<any>;
export { useEventBusWithoutContext as default, EventBus, publish, useSubscribeWithoutContext as useSubscribe };
