"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.useSubscribe = exports.publish = exports.EventBus = exports.default = exports.createContextEventBus = exports.createEventBus = void 0;
var react_1 = require("react");
var useRefCallback_1 = __importDefault(require("../useRefCallback"));
var eventBus_1 = require("./eventBus");
Object.defineProperty(exports, "EventBus", { enumerable: true, get: function () { return eventBus_1.EventBus; } });
/**
 * 订阅当前事件的函数，回调函数传进来后就不会再改变，如果在react组件中中使用有可能会造成函数依赖值不是最新
 * 一般情况下hooks/组件中直接使用useSubscribe，外部js中订阅使用subscribe
 */
function subscribeOrigin(bus, type, callback, options) {
    if (options === void 0) { options = {}; }
    return options.once ? bus === null || bus === void 0 ? void 0 : bus.subscribeOnce(type, callback) : bus === null || bus === void 0 ? void 0 : bus.subscribe(type, callback);
}
function useSubscribeOrigin(bus, eventName, callback, options) {
    if (options === void 0) { options = {}; }
    var subscription = (0, useRefCallback_1.default)(callback);
    (0, react_1.useEffect)(function () {
        var unSubscribe = subscribeOrigin(bus, eventName, subscription, options);
        return function () {
            unSubscribe && unSubscribe();
        };
    }, []);
}
function useEventBus(bus, type) {
    return {
        publish: type ? bus.publish.bind(null, type) : bus.publish.bind(null),
        useSubscribe: type ? useSubscribeOrigin.bind(null, bus, type) : useSubscribeOrigin.bind(null, bus),
        subscribe: type ? subscribeOrigin.bind(null, bus, type) : subscribeOrigin.bind(null, bus),
    };
}
/**
 * 创建一个独立的eventBus实例
 * @returns
 */
function createEventBus() {
    var bus = new eventBus_1.EventBus();
    return {
        publish: bus === null || bus === void 0 ? void 0 : bus.publish.bind(null),
        useSubscribe: useSubscribeOrigin.bind(null, bus),
        subscribe: subscribeOrigin.bind(null, bus),
        useEventBus: useEventBus.bind(null, bus),
    };
}
exports.createEventBus = createEventBus;
/**
 * 可以生成一个用于上下文的EventBus及相关hooks
 */
function createContextEventBus() {
    var contextState = createEventBus();
    var eventBusContext = (0, react_1.createContext)(contextState);
    var useContextEventBus = function () {
        return (0, react_1.useContext)(eventBusContext);
    };
    eventBusContext.useEventBus = useContextEventBus;
    return eventBusContext;
}
exports.createContextEventBus = createContextEventBus;
var _a = createEventBus(), publish = _a.publish, useEventBusWithoutContext = _a.useEventBus, useSubscribeWithoutContext = _a.useSubscribe;
exports.publish = publish;
exports.default = useEventBusWithoutContext;
exports.useSubscribe = useSubscribeWithoutContext;
