"use strict";
var __read = (this && this.__read) || function (o, n) {
    var m = typeof Symbol === "function" && o[Symbol.iterator];
    if (!m) return o;
    var i = m.call(o), r, ar = [], e;
    try {
        while ((n === void 0 || n-- > 0) && !(r = i.next()).done) ar.push(r.value);
    }
    catch (error) { e = { error: error }; }
    finally {
        try {
            if (r && !r.done && (m = i["return"])) m.call(i);
        }
        finally { if (e) throw e.error; }
    }
    return ar;
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.clearTimer = exports.setIntervalBySetTimeout = exports.isFunction = exports.getTimeByString = exports.getDateByString = exports.getformatTimeInfo = exports.formatTime = exports.isBrowser = exports.chunkArray = exports.isObject = exports.isNil = exports.invariant = void 0;
function invariant(condition, message) {
    if (!condition) {
        throw new Error(message);
    }
}
exports.invariant = invariant;
function isNil(value) {
    return value === undefined || value === null;
}
exports.isNil = isNil;
function isObject(value) {
    return typeof value === 'object' && Object.prototype.toString.call(value) === '[object Object]';
}
exports.isObject = isObject;
function chunkArray(array, size) {
    if (size === void 0) { size = 1; }
    var length = (array === null || array === void 0 ? void 0 : array.length) || 0;
    if (!length || size < 1) {
        return [];
    }
    var index = 0;
    var result = [];
    while (index < length) {
        result.push(array.slice(index, (index += size)));
    }
    return result;
}
exports.chunkArray = chunkArray;
exports.isBrowser = !!(typeof window !== 'undefined' && window.document && window.document.createElement);
var formatTime = function (timestamp, format) {
    if (format === void 0) { format = 'dd hh:mm:ss.SSS'; }
    var formatedStr = format;
    var dateFormat = {
        'd+': Math.floor(timestamp / 1000 / 60 / 60 / 24),
        'h+': Math.floor((timestamp / 1000 / 60 / 60) % 24),
        'm+': Math.floor((timestamp / 1000 / 60) % 60),
        's+': Math.floor((timestamp / 1000) % 60),
        'S+': timestamp % 1000,
    };
    Object.entries(dateFormat).forEach(function (_a) {
        var _b = __read(_a, 2), key = _b[0], value = _b[1];
        var reg = new RegExp("(".concat(key, ")"));
        if (reg.test(formatedStr))
            formatedStr = formatedStr.replace(reg, function (m) {
                if (key === 'S+') {
                    return "".concat(value).concat(new Array(m.length).join('0')).slice(0, m.length);
                }
                return m.length === 1 ? "".concat(value) : "00".concat(value).substring("".concat(value).length);
            });
    });
    return formatedStr;
};
exports.formatTime = formatTime;
var getformatTimeInfo = function (timestamp) {
    return {
        day: Math.floor(timestamp / 1000 / 60 / 60 / 24),
        hour: Math.floor((timestamp / 1000 / 60 / 60) % 24),
        minute: Math.floor((timestamp / 1000 / 60) % 60),
        second: Math.floor((timestamp / 1000) % 60),
        millisecond: timestamp % 1000,
    };
};
exports.getformatTimeInfo = getformatTimeInfo;
var TIME_FORMAT_REG = /^(\d{4})[-/]?(\d{1,2})?[-/]?(\d{0,2})[Tt\s]*(\d{1,2})?:?(\d{1,2})?:?(\d{1,2})?[.:]?(\d+)?$/;
var getDateByString = function (date, utc) {
    if (utc === void 0) { utc = false; }
    if (typeof date === 'string' && !/Z$/i.test(date)) {
        var d = date.match(TIME_FORMAT_REG);
        if (d) {
            var m = d[2] - 1 || 0;
            var ms = (d[7] || '0').substring(0, 3);
            if (utc) {
                return new Date(Date.UTC(d[1], m, d[3] || 1, d[4] || 0, d[5] || 0, d[6] || 0, ms));
            }
            return new Date(d[1], m, d[3] || 1, d[4] || 0, d[5] || 0, d[6] || 0, ms);
        }
    }
    return new Date(date);
};
exports.getDateByString = getDateByString;
var getTimeByString = function (date, utc) {
    if (utc === void 0) { utc = false; }
    return (0, exports.getDateByString)(date, utc).getTime();
};
exports.getTimeByString = getTimeByString;
/**
 * isFunction
 */
function isFunction(value) {
    return typeof value === 'function';
}
exports.isFunction = isFunction;
function setIntervalBySetTimeout(callback, delay) {
    var timer = null;
    var offset = 0; // 误差时间
    var count = 0;
    var nextDelay = delay;
    var start = Date.now();
    var loop = function () {
        nextDelay = delay - offset;
        callback();
        timer = setTimeout(function () {
            offset = Date.now() - start - count * delay;
            count++;
            loop();
        }, nextDelay);
    };
    loop();
    return timer;
}
exports.setIntervalBySetTimeout = setIntervalBySetTimeout;
function clearTimer(timer, timeoutType) {
    if (!timer)
        return;
    if (timeoutType === 'raf') {
        try {
            var clearRaf = window.cancelAnimationFrame || window.webkitCancelAnimationFrame;
            clearRaf(timer);
        }
        catch (error) { }
        return;
    }
    if (timeoutType === 'interval') {
        try {
            clearInterval(timer);
        }
        catch (error) { }
        return;
    }
    try {
        clearTimeout(timer);
    }
    catch (error) { }
}
exports.clearTimer = clearTimer;
